from abc import ABC, abstractmethod
from typing import Generic, TypeVar

InputState = TypeVar('InputState')
OutputState = TypeVar('OutputState')


class Step(ABC, Generic[InputState, OutputState]):

    @abstractmethod
    def execute(self, state: InputState) -> OutputState:
        pass

    @abstractmethod
    def compensate(self, state: OutputState) -> None:
        pass



from typing import Tuple, Any, List

from .steps import Step


def _compensate_completed_steps(completed_steps: List[Tuple[Step, Any]]):
    for (step, state) in completed_steps:
        step.compensate(state)


def run_transaction(steps: List[Step], starting_state=None):
    completed_steps: List[Tuple[Step, Any]] = []
    state = starting_state
    try:
        for step in steps:
            state = step.execute(state)
            completed_steps.append((step, state))
        return state

    except Exception as error:
        _compensate_completed_steps(completed_steps)
        return error



from distutils.core import setup
setup(
    name = "talepy",
    packages = ["talepy"],
    version = "0.1.0",
    description = "Distributed Transactions Helper",
    author = "Steve Brazier",
    author_email = "meadsteve@gmail.com",
    url = "https://github.com/meadsteve/talepy",
    keywords = ["saga", "distributed transaction"],
    classifiers = [
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Development Status :: 4 - Beta",
        ],
    long_description = """\
Tale Distributed Transactions
-------------------------------------

"""
)